@@ -96,12 +96,16 @@ def spawn_raw(function, *args, **kwargs):
        if ``GEVENT_TRACK_GREENLET_TREE`` is enabled (the default). If not enabled,
        those attributes will not be set.
 
+    .. versionchanged:: 1.5a3
+       The returned greenlet always has a *loop* attribute matching the
+       current hub's loop. This helps it work better with more gevent APIs.
     """
     if not callable(function):
         raise TypeError("function must be callable")
 
     # The hub is always the parent.
     hub = _get_hub_noargs()
+    loop = hub.loop
 
     factory = TrackedRawGreenlet if GEVENT_CONFIG.track_greenlet_tree else RawGreenlet
 
@@ -111,11 +115,11 @@ def spawn_raw(function, *args, **kwargs):
     if kwargs:
         function = _functools_partial(function, *args, **kwargs)
         g = factory(function, hub)
-        hub.loop.run_callback(g.switch)
+        loop.run_callback(g.switch)
     else:
         g = factory(function, hub)
-        hub.loop.run_callback(g.switch, *args)
-
+        loop.run_callback(g.switch, *args)
+    g.loop = hub.loop
     return g
 
 
@@ -152,6 +156,10 @@ def sleep(seconds=0, ref=True):
         waiter.get()
     else:
         with loop.timer(seconds, ref=ref) as t:
+            # Sleeping is expected to be an "absolute" measure with
+            # respect to time.time(), not a relative measure, so it's
+            # important to update the loop's notion of now before we start
+            loop.update_now()
             hub.wait(t)
 
 
@@ -167,10 +175,10 @@ def idle(priority=0):
     .. seealso:: :func:`sleep`
     """
     hub = _get_hub_noargs()
-    watcher = hub.loop.idle()
-    if priority:
-        watcher.priority = priority
-    hub.wait(watcher)
+    with hub.loop.idle() as watcher:
+        if priority:
+            watcher.priority = priority
+        hub.wait(watcher)
 
 
 def kill(greenlet, exception=GreenletExit):
@@ -205,25 +213,32 @@ def kill(greenlet, exception=GreenletExit):
 
 class signal(object):
     """
+    signal_handler(signalnum, handler, *args, **kwargs) -> object
+
     Call the *handler* with the *args* and *kwargs* when the process
     receives the signal *signalnum*.
 
-    The *handler* will be run in a new greenlet when the signal is delivered.
+    The *handler* will be run in a new greenlet when the signal is
+    delivered.
 
-    This returns an object with the useful method ``cancel``, which, when called,
-    will prevent future deliveries of *signalnum* from calling *handler*.
+    This returns an object with the useful method ``cancel``, which,
+    when called, will prevent future deliveries of *signalnum* from
+    calling *handler*.
 
     .. note::
 
-        This may not operate correctly with SIGCHLD if libev child watchers
-        are used (as they are by default with os.fork).
+        This may not operate correctly with ``SIGCHLD`` if libev child
+        watchers are used (as they are by default with
+        `gevent.os.fork`). See :mod:`gevent.signal` for a more
+        general purpose solution.
 
     .. versionchanged:: 1.2a1
-       The ``handler`` argument is required to be callable at construction time.
-    """
 
-    # XXX: This is manually documented in gevent.rst while it is aliased in
-    # the gevent module.
+        The ``handler`` argument is required to
+        be callable at construction time.
+    """
+    # This is documented as a function, not a class,
+    # so we're free to change implementation details.
 
     greenlet_class = None
 
@@ -335,8 +350,6 @@ def reinit(hub=None):
     #sleep(0.00001)
 
 
-hub_ident_registry = IdentRegistry()
-
 class Hub(WaitOperationsGreenlet):
     """
     A greenlet that runs the event loop.
@@ -382,6 +395,15 @@ class Hub(WaitOperationsGreenlet):
     # because that conflicts with the slot we inherit from the
     # Cythonized-bases.
 
+    # This is the source for our 'minimal_ident' property. We don't use a
+    # IdentRegistry because we've seen some crashes having to do with
+    # clearing weak references on shutdown in Windows (see known_failures.py).
+    # This gives us slightly different semantics than a greenlet's minimal_ident
+    # (notably, there can be holes) but we never documented this object's minimal_ident,
+    # and there should be few enough hub's over the lifetime of a process so as not
+    # to matter much.
+    _hub_counter = 0
+
     def __init__(self, loop=None, default=None):
         WaitOperationsGreenlet.__init__(self, None, None)
         self.thread_ident = get_thread_ident()
@@ -404,7 +426,9 @@ class Hub(WaitOperationsGreenlet):
         self._resolver = None
         self._threadpool = None
         self.format_context = GEVENT_CONFIG.format_context
-        self.minimal_ident = hub_ident_registry.get_ident(self)
+
+        Hub._hub_counter += 1
+        self.minimal_ident = Hub._hub_counter
 
     @Lazy
     def ident_registry(self):
@@ -447,25 +471,47 @@ class Hub(WaitOperationsGreenlet):
         result += ' thread_ident=%s' % (hex(self.thread_ident), )
         return result + '>'
 
-    def handle_error(self, context, type, value, tb):
-        """
-        Called by the event loop when an error occurs. The arguments
-        type, value, and tb are the standard tuple returned by :func:`sys.exc_info`.
+    def _normalize_exception(self, t, v, tb):
+        # Allow passing in all None if the caller doesn't have
+        # easy access to sys.exc_info()
+        if (t, v, tb) == (None, None, None):
+            t, v, tb = sys.exc_info()
 
-        Applications can set a property on the hub with this same signature
-        to override the error handling provided by this class.
-
-        Errors that are :attr:`system errors <SYSTEM_ERROR>` are passed
-        to :meth:`handle_system_error`.
-
-        :param context: If this is ``None``, indicates a system error that
-            should generally result in exiting the loop and being thrown to the
-            parent greenlet.
-        """
-        if isinstance(value, str):
+        if isinstance(v, str):
             # Cython can raise errors where the value is a plain string
             # e.g., AttributeError, "_semaphore.Semaphore has no attr", <traceback>
-            value = type(value)
+            v = t(v)
+
+        return t, v, tb
+
+    def handle_error(self, context, type, value, tb):
+        """
+        Called by the event loop when an error occurs. The default
+        action is to print the exception to the :attr:`exception
+        stream <exception_stream>`.
+
+        The arguments ``type``, ``value``, and ``tb`` are the standard
+        tuple as returned by :func:`sys.exc_info`. (Note that when
+        this is called, it may not be safe to call
+        :func:`sys.exc_info`.)
+
+        Errors that are :attr:`not errors <NOT_ERROR>` are not
+        printed.
+
+        Errors that are :attr:`system errors <SYSTEM_ERROR>` are
+        passed to :meth:`handle_system_error` after being printed.
+
+        Applications can set a property on the hub instance with this
+        same signature to override the error handling provided by this
+        class. This is an advanced usage and requires great care. This
+        function *must not* raise any exceptions.
+
+        :param context: If this is ``None``, indicates a system error
+            that should generally result in exiting the loop and being
+            thrown to the parent greenlet.
+        """
+        type, value, tb = self._normalize_exception(type, value, tb)
+
         if not issubclass(type, self.NOT_ERROR):
             self.print_exception(context, type, value, tb)
         if context is None or issubclass(type, self.SYSTEM_ERROR):
@@ -500,28 +546,39 @@ class Hub(WaitOperationsGreenlet):
     def exception_stream(self):
         """
         The stream to which exceptions will be written.
-        Defaults to ``sys.stderr`` unless assigned to.
+        Defaults to ``sys.stderr`` unless assigned. Assigning a
+        false (None) value disables printing exceptions.
 
         .. versionadded:: 1.2a1
         """
         # Unwrap any FileObjectThread we have thrown around sys.stderr
         # (because it can't be used in the hub). Tricky because we are
         # called in error situations when it's not safe to import.
-        stderr = sys.stderr
+        # Be careful not to access sys if we're in the process of interpreter
+        # shutdown.
+        stderr = sys.stderr if sys else None # pylint:disable=using-constant-test
         if type(stderr).__name__ == 'FileObjectThread':
             stderr = stderr.io # pylint:disable=no-member
         return stderr
 
-    def print_exception(self, context, type, value, tb):
+    def print_exception(self, context, t, v, tb):
         # Python 3 does not gracefully handle None value or tb in
         # traceback.print_exception() as previous versions did.
         # pylint:disable=no-member
         errstream = self.exception_stream
+        if not errstream: # pragma: no cover
+            # If the error stream is gone, such as when the sys dict
+            # gets cleared during interpreter shutdown,
+            # don't cause follow-on errors.
+            # See https://github.com/gevent/gevent/issues/1295
+            return
 
-        if value is None:
-            errstream.write('%s\n' % type.__name__)
+        t, v, tb = self._normalize_exception(t, v, tb)
+
+        if v is None:
+            errstream.write('%s\n' % t.__name__)
         else:
-            traceback.print_exception(type, value, tb, file=errstream)
+            traceback.print_exception(t, v, tb, file=errstream)
         del tb
 
         try:
@@ -539,7 +596,7 @@ class Hub(WaitOperationsGreenlet):
                 except: # pylint:disable=bare-except
                     traceback.print_exc(file=self.exception_stream)
                     context = repr(context)
-            errstream.write('%s failed with %s\n\n' % (context, getattr(type, '__name__', 'exception'), ))
+            errstream.write('%s failed with %s\n\n' % (context, getattr(t, '__name__', 'exception'), ))
 
 
     def run(self):
