@@ -164,14 +164,15 @@ disabled_tests = [
     # the file to a temp location during patching.
     'test_asyncore.HelperFunctionTests.test_compact_traceback',
 
-    'test_signal.WakeupSignalTests.test_wakeup_fd_early',
     # expects time.sleep() to return prematurely in case of a signal;
-    # gevent.sleep() is better than that and does not get interrupted (unless signal handler raises an error)
+    # gevent.sleep() is better than that and does not get interrupted
+    # (unless signal handler raises an error)
+    'test_signal.WakeupSignalTests.test_wakeup_fd_early',
 
+    # expects select.select() to raise select.error(EINTR'interrupted
+    # system call') gevent.select.select() does not get interrupted
+    # (unless signal handler raises an error) maybe it should?
     'test_signal.WakeupSignalTests.test_wakeup_fd_during',
-    # expects select.select() to raise select.error(EINTR'interrupted system call')
-    # gevent.select.select() does not get interrupted (unless signal handler raises an error)
-    # maybe it should?
 
     'test_signal.SiginterruptTest.test_without_siginterrupt',
     'test_signal.SiginterruptTest.test_siginterrupt_on',
@@ -709,10 +710,17 @@ if PYPY:
         # These are flaxy, apparently a race condition? Began with PyPy 2.7-7 and 3.6-7
         'test_asyncore.TestAPI_UsePoll.test_handle_error',
         'test_asyncore.TestAPI_UsePoll.test_handle_read',
-
-
     ]
 
+    if WIN:
+        disabled_tests += [
+            # Starting in 7.3.1 on Windows, this stopped raising ValueError; it appears to
+            # be a bug in PyPy.
+            'test_signal.WakeupFDTests.test_invalid_fd',
+            # Likewise for 7.3.1. See the comments for PY35
+            'test_socket.GeneralModuleTests.test_sock_ioctl',
+        ]
+
     if PY36:
         disabled_tests += [
             # These are flaky, beginning in 3.6-alpha 7.0, not finding some flag
@@ -1239,6 +1247,13 @@ disabled_tests += [
     'test_ssl.BasicSocketTests.test_openssl_version'
 ]
 
+if OSX:
+
+    disabled_tests += [
+        # This sometimes produces OSError: Errno 40: Message too long
+        'test_socket.RecvmsgIntoTCPTest.testRecvmsgIntoGenerator',
+    ]
+
 # Now build up the data structure we'll use to actually find disabled tests
 # to avoid a linear scan for every file (it seems the list could get quite large)
 # (First, freeze the source list to make sure it isn't modified anywhere)
